import os
import sys
from multiprocessing import Queue, Process, cpu_count
import gzip
import filetype
import pandas as pd
# import shutil
# import tarfile

__author__ = "Anupam_Gautam"


class CombinerClass:

    def __init__(self,TakeFileDirectory,TakeCounter,TakeOrganismNameToSend,TakeOutPutFileName,TakeThread):

        self.TakeFileDirectory=TakeFileDirectory
        self.TakeCounter=TakeCounter
        self.TakeOrganismNameToSend=TakeOrganismNameToSend
        self.TakeOutPutFileName=TakeOutPutFileName
        self.TakeThread=TakeThread
    
    def getFileList(self,DataFrame):
        '''
        Returns the list of files in specified folder.
        '''
        filelist = []
        counter=self.TakeCounter
        for value in DataFrame.iloc[:,0].to_list():
            counter=counter+1
            filepath=os.path.abspath(os.path.join(self.TakeFileDirectory,value))
            filepath=str(counter)+"&"+filepath
            filelist.append(filepath)
        
        return filelist

    def Parallelizer(self,ListOffiles):
        q = Queue()
        procs = []
        for i in range(0,int(self.TakeThread)):
        # Split the source filelist into several sublists.
            lst = [ListOffiles[j] for j in range(0, len(ListOffiles)) if j % int(self.TakeThread) == i]
            if len(lst)>0:
                #p = Process(target=FastaDetailProcessor, args=([lst, q,RecieveFilepath]))
                p = Process(target=self.FastaFileProcessor , args=([lst,self.TakeOrganismNameToSend,self.TakeOutPutFileName,q]))
                p.start()
                procs += [p]
        # Collect the results:
        all_results = []
        for i in range(0, len(procs)):
        # Save all results from the queue.
            all_results += q.get()

        return(len(ListOffiles))

    @staticmethod
    def FastaFileProcessor(TakeFileNameString,TakeOrganismNameToUse,TakeOutputFileName,q):
        try:
            result=[]
        #workQueue='done'
            for NameOfFile in TakeFileNameString:
                fileName=NameOfFile.split("&")
                orgNumber=fileName[0]
                HeaderFileName=TakeOutputFileName+".Header"
                OrgListFileName=TakeOutputFileName+".Orglist"
                fw=open(TakeOutputFileName+"_InputAll.faa", "a")
                fwHeader=open(HeaderFileName, "a")
                fwOrg=open(OrgListFileName, "a")
                fwOrg.write(str(TakeOrganismNameToUse)+str(orgNumber)+"\t"+str(fileName[1])+"\n")
                kind = filetype.guess(fileName[1])
                #print(kind)
                if kind is not None:
                    if kind.extension in ["gz","bz2"]:
                        f  = gzip.open(fileName[1], "rt")
                    #print(fileName)
                    #ProcessGzFile(NameOfFile,TakeOrganismNameToUse,TakeOutputFileName)
                elif kind is None:
                    # print("hello")
                    f  = open(fileName[1], "r")
                    #ProcessNormalFile(NameOfFile,TakeOrganismNameToUse,TakeOutputFileName)
                firstLine = f.readline()
                Header=firstLine.strip("\n")
                counter=0
                sequence=''
                for line in f:
                    line=line.strip("\n")
                    if '>' in line:
                        counter=counter+1
                        fw.write('>'+str(TakeOrganismNameToUse)+str(orgNumber)+"|"+str(TakeOrganismNameToUse)+str(orgNumber)+"_"+str('gene')+str(counter)+"\n"+str(sequence)+"\n")
                        fwHeader.write('>'+str(TakeOrganismNameToUse)+str(orgNumber)+"|"+str(TakeOrganismNameToUse)+str(orgNumber)+"_"+str('gene')+str(counter)+"\t"+str(fileName[1])+"\t"+str(Header)+"\n")
                        Header=line
                        sequence=''
                    else:
                        sequence=sequence+line
                fw.write('>'+str(TakeOrganismNameToUse)+str(orgNumber)+"|"+str(TakeOrganismNameToUse)+str(orgNumber)+"_"+str('gene')+str(counter)+"\n"+str(sequence)+"\n")
                fwHeader.write('>'+str(TakeOrganismNameToUse)+str(orgNumber)+"|"+str(TakeOrganismNameToUse)+str(orgNumber)+"_"+str('gene')+str(counter)+"\t"+str(fileName[1])+"\t"+str(Header)+"\n")
                sequence=''
                f.close()
                fw.close()
                fwOrg.close()
                fwHeader.close()
            result.append("Done")
        except:
            q.put([])
            raise
        q.put(result)

def CombinerEntry(ReceiveFastaFileDirectory: str="", ReceiveOutputDirectory: str="",ReceiveMetadatfile: str="",ReceiveColumnName: str="", ReceivePrefixForFilename: str="", ReceiveThread: int="") -> None:

    ReceiveOutPutFileName=os.path.abspath(os.path.join(str(ReceiveOutputDirectory),str(ReceivePrefixForFilename)))

    metadatafileInitial=pd.read_csv(ReceiveMetadatfile,sep="\t")
    metadata=metadatafileInitial[['#SampleID',ReceiveColumnName]]
    name=metadata[ReceiveColumnName].unique()
    grouped = [x for _, x in metadata.groupby(ReceiveColumnName)]
    iteratorCount=0
    for numberofGroup in range(0,len(grouped)):

        iteratorCount=iteratorCount+1

        if iteratorCount==1:
            counter=0
        else:
            counter=NumberOfFile

        CombinerClassHandler=CombinerClass(TakeFileDirectory=ReceiveFastaFileDirectory,TakeCounter=counter, TakeOrganismNameToSend=str(name[numberofGroup]+"_Org"), TakeOutPutFileName=ReceiveOutPutFileName, TakeThread=ReceiveThread)
        getFileListReturn=CombinerClassHandler.getFileList(grouped[numberofGroup])
        NumberOfFile=CombinerClassHandler.Parallelizer(getFileListReturn)

    return(ReceiveOutPutFileName+"_InputAll.faa")
    # InitialProcessHandler=InitialProcess(ReceiveFastaFileDirectory,ReceiveOutPutFileName,ReceiveMetadatfile,ReceiveThread)
    # files = InitialProcessHandler.getFileList()

    # #print(files)
    # logging.info('Total Number of Files '+ str(len(files)))
    # NumberOfFile=InitialProcessHandler.Parallelizer(files)

    # tarDir(PathOfDir).tarIt()




from optparse import OptionParser, OptionGroup
import sys
import os
from multiprocessing import Queue, Process, cpu_count
from metapan.modules.panning import panningEntry


__author__ = "Anupam_Gautam"

# def mainFunction(args):
#     print(hello)
def main():
    parser = OptionParser("%prog [options] infile",
                          description="Run pangenome and does comparison at Functional level",
                          epilog=__author__)


    parser.add_option("-i", "--inputfiledirectory", default="", action="store", dest="inputfiledirectory",
                       help="Input file directory",metavar="inputfiledirectory")
    parser.add_option("-o", "--output",default="metapan", action="store", dest="output",
                      help="output folder path [Default: metapan]", metavar="output" )
    parser.add_option("-m", "--metadatafile", action="store",dest="metadatafile",
                       help="Enter name for input File",metavar="metadatafile")
    parser.add_option("-g", "--columnname", action="store",dest="columnname",
                       help="Enter columnname name for grouping",metavar="columnname")
    cluster = OptionGroup(parser,"Clustering Parameter")
    # cluster.add_argument('-m',  metavar='MatrixFile',dest='MatrixFile', required=True,
    #                    help='Enter name for matrix file')
    cluster.add_option("-s", "--tool", default='cdhit', action="store",dest="tool", 
                       help="Enter tool to use for clustering cdhit, usearch etc [Default: cdhit]",metavar="tool")
    cluster.add_option("--toolpath", default='', action="store",dest="toolpath", 
                       help="Enter path to tool to use for clustering cdhit, usearch etc",metavar="toolpath")
    cluster.add_option("-c", "--sequenceidentity", default=0.9, action="store",dest="sequenceidentity", 
                       help="Enter sequence identity threshold, [Default: 0.9]",metavar="sequenceidentity")
    cluster.add_option("-w", "--wordlength", default=5 , action="store",dest="wordlength",  
                       help="Enter word_length, see CD-HIT  user guide for choosing it [Default: 5]", metavar="wordlength")
    cluster.add_option("-a","--alignmentcoverage", dest="alignmentcoverage", action="store", default=0.9,
                       help='Enter alignment coverage for the longer sequence, if set to 0.9 the alignment must covers 90%% of the sequence [Default: 0.9]',metavar='alignmentcoverage')
    parser.add_option_group(cluster)


    GeneralParameter = OptionGroup(parser, "General Parameter")
    GeneralParameter.add_option('-p',"--prefix",action="store",dest='prefix', default="prefix",
                       help='Enter prefix', metavar='prefix')
    GeneralParameter.add_option('-t',"--threads",action="store",dest='threads', default=cpu_count(),
                       help='Enter number of threads', metavar='threads')
    GeneralParameter.add_option('-r',"--ram",action="store",dest='ram', default="4gb",
                       help='Enter ram in gb you want to use [Default: 2gb]',metavar='ram')
    # parser.add_option(GeneralParameter)

    parser.add_option_group(GeneralParameter)
    (options, args) = parser.parse_args()

    # if len(args) == 1:
    #     infile = args[0]
    # elif len(args) == 0:
    #     raise IOError("Must specify input file (use - for stdin)")
    # else:
    #     raise IOError("Too many arguments", args)


    panningEntry(inputfiledirectory=options.inputfiledirectory, output=options.output, metadatfile=options.metadatafile, columnname=options.columnname, tool=options.tool, toolpath=options.toolpath, sequenceidentity=options.sequenceidentity, wordlength=options.wordlength, alignmentcoverage=options.alignmentcoverage, prefix=options.prefix,threads=options.threads, ram=options.ram)

if __name__ == '__main__':
	main()



import sys
import time
import datetime
import tarfile
import gzip
import shutil
import os



def gb_to_mb(gb):
    #######convert gb to mb
	return(int(gb.replace("gb",""))*1024)

class tarDir:
        def __init__(self,InputDirectoryPathToTar):

            self.InputDirectoryPathToTar=InputDirectoryPathToTar


        def tarIt(self):
            tar = tarfile.open(str(self.InputDirectoryPathToTar)+".tar.gz", "w:gz")
            tar.add(str(self.InputDirectoryPathToTar), arcname=os.path.basename(os.path.abspath(self.InputDirectoryPathToTar.strip("\n"))))
            tar.close()
            shutil.rmtree(str(self.InputDirectoryPathToTar), ignore_errors=True)

class OutputDirectoryGenerator:
 
    def __init__(self, TakeDir,TakeMode):
        self.TakeDir = TakeDir
        self.TakeMode = TakeMode

    def DicGenAndCheck(self):
        if not self.TakeDir:
            # print(os.path.abspath(os.getcwd()))
            x=datetime.datetime.now()
            x=str(x).replace(" ","_").replace("/", "")
            NameDir=self.TakeMode+x
            ModeDir=os.path.join(os.path.abspath(os.getcwd()),NameDir)
            os.mkdir(ModedDir)
            return(ModeDir)
        else:
            if os.path.isdir(os.path.abspath(self.TakeDir)):
                x=datetime.datetime.now()
                x=str(x).replace(" ","_").replace("/", "")
                NameDir=self.TakeMode+x
                ModeDir=os.path.join(os.path.abspath(self.TakeDir),NameDir)
                os.mkdir(ModeDir)
                return(ModeDir)
            else:
                os.mkdir(os.path.abspath(self.TakeDir))
                x=datetime.datetime.now()
                x=str(x).replace(" ","_").replace("/", "")
                NameDir=self.TakeMode+x
                ModeDir=os.path.join(os.path.abspath(self.TakeDir),NameDir)
                os.mkdir(ModeDir)
                return(ModeDir)


import pandas as pd 
import numpy as np
import os
from metapan.modules.utility import OutputDirectoryGenerator,tarDir,gb_to_mb
from metapan.modules.combiner import CombinerEntry

class pann:

    def __init__(self,inputfiledirectory, outputPath, metadatfile, columnname, tool, toolpath, sequenceidentity, wordlength, alignmentcoverage, prefix, threads,ram):

        self.inputfiledirectory = inputfiledirectory
        self.outputPath = outputPath
        self.metadatfile = metadatfile
        self.columnname = columnname
        self.tool = tool
        self.toolpath = toolpath
        self.sequenceidentity = sequenceidentity
        self.wordlength = wordlength
        self.alignmentcoverage = alignmentcoverage
        self.prefix= prefix
        self.threads= threads
        self.ram= ram

    def fileProcessing(self):
        combinedFastaFile=CombinerEntry(ReceiveFastaFileDirectory=self.inputfiledirectory, ReceiveOutputDirectory=self.outputPath, ReceiveMetadatfile=self.metadatfile,ReceiveColumnName=self.columnname,ReceivePrefixForFilename =self.prefix, ReceiveThread=self.threads)
        return(combinedFastaFile)

    def ForCluster(self,TakeFastaFile):

        ram=gb_to_mb(self.ram)
        if self.toolpath:
            if self.tool=='cdhit':
                tool=os.path.abspath(self.toolpath)
            elif self.tool=='usearch':
                tool='usearch'
        else:
            if self.tool=='cdhit':
                tool='cdhit'
            elif self.tool=='usearch':
                tool='usearch'
    # got a non-empty string

        if self.tool =='cdhit':
            stringCdHit=tool+' -i '+TakeFastaFile.strip("\n")+' -o '+TakeFastaFile.replace("_InputAll.faa","")+'   -c '+str(self.sequenceidentity)+' -T '+str(self.threads)+' -n '+str(self.wordlength)+' -d 300 -aL '+str(self.alignmentcoverage)+' -sc 1  -sf 1 -bak 1 -M '+str(ram)#cdhit command
            #print(stringCdHit)
            os.system(stringCdHit)

def panningEntry(inputfiledirectory: str ="", output: str = "metapan", metadatfile: str="",columnname: str="", tool: str ="", toolpath: str="", sequenceidentity: float=0.9, wordlength: int=5, alignmentcoverage: float=0.9, prefix: str="",threads: int="",ram: str = "") -> None:

    DirectoryCaller=OutputDirectoryGenerator(os.path.abspath(output),'metapan')
    PathOfDir=DirectoryCaller.DicGenAndCheck()
    # print(inputfiledirectory, PathOfDir, os.path.abspath(metadatfile), columnname,sequenceidentity, wordlength, alignmentcoverage, prefix,threads,ram)
    pannHandler=pann(inputfiledirectory, PathOfDir, os.path.abspath(metadatfile), columnname, tool, toolpath, sequenceidentity, wordlength, alignmentcoverage, prefix,threads,ram)
    combinedFastaFile=pannHandler.fileProcessing()
    pannHandler.ForCluster(combinedFastaFile)


